package com.perforce.p4java.common.io;

import com.perforce.p4java.common.base.Utf8Utils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.CharacterCodingException;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CodingErrorAction;
import java.nio.charset.StandardCharsets;
import java.util.Arrays;

import javax.annotation.Nonnull;

import static com.perforce.p4java.common.quantity.Amount.DEFAULT_BUFFER_SIZE;

/**
 * @author Sean Shou
 * @since 25/08/2016
 */
public class FileEncoderConverter {
    private FileEncoderConverter() { /* util */ }

    public static boolean canBeConvertWellFormedUtf8(
            @Nonnull final byte[] sourceBytes,
            final Charset from) {

        try {
            byte[] bytes = convertBytesEncode(sourceBytes, from, StandardCharsets.UTF_8);
            return Utf8Utils.isWellFormed(bytes);
        } catch (CharacterCodingException e) {
            return false;
        }
    }

    public static boolean canBeConvertWellFormedUtf8(
            @Nonnull final File file,
            final Charset from) throws IOException {

        try (InputStream fileInputStream = new FileInputStream(file)) {
            byte[] buffer = new byte[DEFAULT_BUFFER_SIZE];
            int bytesRead;
            while ((bytesRead = fileInputStream.read(buffer)) > 0) {
                byte[] readReadBytes = Arrays.copyOf(buffer, bytesRead);
                boolean convertable = canBeConvertWellFormedUtf8(readReadBytes, from);
                if (!convertable) {
                    return false;
                }
            }
            return true;
        }
    }

    public static byte[] convertBytesEncode(
            @Nonnull final byte[] sourceBytes,
            final Charset from,
            final Charset to) throws CharacterCodingException {

        CharsetDecoder sourceDecoder = from.newDecoder();
        ByteBuffer decodedByteBuffer = ByteBuffer.wrap(sourceBytes);
        CharBuffer sourceCharBuffer = sourceDecoder.decode(decodedByteBuffer);

        CharsetEncoder targetEncoder = to.newEncoder();
        targetEncoder.onMalformedInput(CodingErrorAction.REPORT);
        targetEncoder.onUnmappableCharacter(CodingErrorAction.REPORT);
        ByteBuffer encodedByteBuffer = targetEncoder.encode(sourceCharBuffer);
        byte[] targetEncodedBytes = Arrays.copyOf(
                encodedByteBuffer.array(),
                encodedByteBuffer.remaining());
        encodedByteBuffer.clear();
        return targetEncodedBytes;
    }
}
