/**
 *
 */
package com.perforce.p4java.tests.dev.unit.features111;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.core.IsNull.notNullValue;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.util.List;
import java.util.Properties;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.file.FileSpecBuilder;
import com.perforce.p4java.core.file.IFileAnnotation;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.option.server.GetFileAnnotationsOptions;
import com.perforce.p4java.option.server.LoginOptions;
import com.perforce.p4java.server.ServerFactory;
import com.perforce.p4java.tests.dev.annotations.TestId;
import com.perforce.p4java.tests.dev.unit.P4JavaTestCase;

/**
 * Tests annotate -I option. A bit impressionistic. May fail on
 * newly-(re)created servers; if so, play with some of the size constants,
 * etc., below and see what happens...
 */

@TestId("Features111_IntegrationsAnnotationsTest")
public class IntegrationsAnnotationsTest extends P4JavaTestCase {
    private static final int TIME_OUT_IN_SECONDS = 60;

    @Before
    public void beforeEach() throws Exception {
        Properties rpcTimeOutProperties = configRpcTimeOut("IntegrationsAnnotationsTest", TIME_OUT_IN_SECONDS);

        server = getServer(P4JTEST_SERVER_URL_DEFAULT, rpcTimeOutProperties, null, null);
        assertThat(server, notNullValue());
    }

    @Test
    public void testIntegrationsAnnotations() throws Exception {
        final String testRoot = "//depot/102Dev/CopyFilesTest/tgt"; // shared, so be careful...
        final String testFile = testRoot + "/" + "...";
        final List<IFileSpec> testFiles = FileSpecBuilder.makeFileSpecList(testFile);

        IClient client = getDefaultClient(server);
        assertNotNull("null client returned", client);
        server.setCurrentClient(client);
        List<IFileSpec> syncFiles = this.forceSyncFiles(client, testRoot + "/...");
        assertNotNull("null sync files list", syncFiles);
        assertEquals("bad forced sync", 0, FileSpecBuilder.getInvalidFileSpecs(syncFiles).size());
        int annotationsSize = 0;
        List<IFileAnnotation> annotations = server.getFileAnnotations(testFiles, new GetFileAnnotationsOptions());
        assertNotNull("null annotations list returned", annotations);
        annotationsSize = annotations.size();
        assertTrue("annotations list too small", annotationsSize > 300); // rough size...
        for (IFileAnnotation annotation : annotations) {
            assertNotNull("null annotation in list", annotation);
            assertNull("non-null integrations list", annotation.getAllIntegrations());
        }

        annotations = server.getFileAnnotations(testFiles,
                new GetFileAnnotationsOptions().setFollowAllIntegrations(true));
        assertNotNull("null annotations list returned", annotations);
        assertEquals("annotation list size mismatch", annotationsSize, annotations.size()); // Should match
        boolean foundContributing = false;
        boolean foundSpecificContributer = false;
        for (IFileAnnotation annotation : annotations) {
            assertNotNull("null annotation in list", annotation);
            List<IFileAnnotation> contributingAnnotations = annotation.getAllIntegrations();
            if (contributingAnnotations != null) {
                foundContributing = true;
                int i = 0;
                for (IFileAnnotation contAnnotation : contributingAnnotations) {
                    assertNotNull("null contributing annotation", contAnnotation);
                    assertEquals("ordering mismatch", i, contAnnotation.getOrdering());
                    assertNotNull("null depot path in contributing annotation", contAnnotation.getDepotPath());
                    assertNull("non-null data line in contributing annotation", contAnnotation.getLine());
                    i++;
 
						/*
                         * Look for specific integration we know exists:
						 * ... depotFile1 //depot/102Dev/CopyFilesTest/tgt/test01.txt
						 * ... upper1 121
						 * ... lower1 108
						 */

                    if ("//depot/102Dev/CopyFilesTest/tgt/test01.txt".equals(
                            contAnnotation.getDepotPath())
                            && (contAnnotation.getOrdering() == 1)
                            && (contAnnotation.getLower() == 108)
                            && (contAnnotation.getUpper() == 121)) {
                        foundSpecificContributer = true;
                    }
                }
            }
        }
        assertTrue("found no contributing annotations", foundContributing);
        assertTrue("did not find specific contributing annotation", foundSpecificContributer);
    }

    @After
    public void afterEach() throws Exception {
        if (server != null) {
            this.endServerSession(server);
        }
    }
}
