package com.perforce.p4java.tests.dev.unit.features171;

import com.perforce.p4java.PropertyDefs;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.impl.mapbased.client.Client;
import com.perforce.p4java.impl.mapbased.client.ViewDepotType;
import com.perforce.p4java.tests.GraphServerRule;
import com.perforce.p4java.tests.dev.annotations.TestId;
import com.perforce.p4java.tests.dev.unit.P4JavaRshTestCase;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.ClassRule;
import org.junit.Test;

import java.util.Properties;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

/**
 *
 */
@TestId("Dev171_GraphClientDepotTypeTest")
public class GraphClientDepotTypeTest extends P4JavaRshTestCase {

	@ClassRule
	public static GraphServerRule p4d = new GraphServerRule("r17.1", GraphClientDepotTypeTest.class.getSimpleName());

	@BeforeClass
	public static void beforeAll() throws Exception {
		Properties properties = new Properties();
		properties.put(PropertyDefs.IGNORE_FILE_NAME_KEY_SHORT_FORM, ".p4ignore");
		properties.put(PropertyDefs.ENABLE_GRAPH_SHORT_FORM, "true");
		setupServer(p4d.getRSHURL(), "p4jtestsuper", "p4jtestsuper", true, properties);
	}

	@Before
	public void setUp() {
		// initialization code (before each test).
		try {
			client = server.getClient(getPlatformClientName("GraphCatFile.ws"));
			assertNotNull(client);
			server.setCurrentClient(client);
		} catch (P4JavaException e) {
			fail("Unexpected exception: " + e.getLocalizedMessage());
		}
	}

	/**
	 * Tests the case where the server returns a valid depot type
	 *
	 * @throws P4JavaException
	 */
	@Test
	public void testDepotTypeIfServerReturnsAvalidValue() throws P4JavaException {
		IClient client = server.getClient("GraphCatFile.ws");
		assertNotNull(client);
		assertEquals(ViewDepotType.GRAPH, client.getViewDepotType());

		client = server.getClient("p4jtestsuper.ws");
		assertNotNull(client);
		assertEquals(ViewDepotType.HYBRID, client.getViewDepotType());
	}

	/**
	 * Tests the case where server returns no depot type.
	 * In this cas the system reverts to the default depot type of ViewDepotType.LOCAL
	 *
	 * @throws P4JavaException
	 */
	@Test
	public void testDepotTypeIfServerReturnsNull() throws P4JavaException {

		String[] paths = {"//depot/... //local-client/..."};
		IClient testClient = Client.newClient(server, "local-client", "testing depot type", "/var/temp/local-client", paths);
		server.createClient(testClient);

		IClient testClientFromServer = server.getClient("local-client");
		assertNotNull(testClientFromServer);
		assertEquals(ViewDepotType.LOCAL, testClientFromServer.getViewDepotType());
	}

	/**
	 * Tests the case for 'Type' field
	 *
	 * @throws P4JavaException
	 */
	@Test
	public void testDepotTypeField() throws P4JavaException {
		IClient client = server.getClient("GraphCatFile.ws");
		assertNotNull(client);
		assertEquals(null, client.getType());

		client.setType("graph");
		client.update();

		client = server.getClient("GraphCatFile.ws");
		assertEquals("graph", client.getType());
	}
}
